@@ -1,10 +1,12 @@
+from typing import Any, List, Literal, Tuple, Union
+
 import numpy as np
 import xarray as xr
-from pydantic import BaseModel, Extra, Field
-from typing import Literal
 from numpy.typing import NDArray
-from .base import QualityHandler
+from pydantic import BaseModel, Extra, Field, validator
 
+from ..utils import record_corrections_applied
+from .base import QualityHandler
 
 __all__ = [
     "DataQualityError",
@@ -14,20 +16,6 @@ __all__ = [
     "SortDatasetByCoordinate",
 ]
 
-# def record_correction(self, variable_name: str):
-#     """If a correction was made to variable data to fix invalid values
-#     as detected by a quality check, this method will record the fix
-#     to the appropriate variable attribute.  The correction description
-#     will come from the handler params which get set in the pipeline config
-#     file.
-
-#     :param variable_name: Name
-#     :type variable_name: str
-#     """
-#     correction = self.params.get("correction", None)
-#     if correction:
-#         utils.record_corrections_applied(self.ds, variable_name, correction)
-
 
 class DataQualityError(ValueError):
     """Raised when the quality of a variable indicates a fatal error has occurred.
@@ -54,21 +42,53 @@ class FailPipeline(QualityHandler):
         context: str = ""
         """Additional context set by users that ends up in the traceback message."""
 
+        display_limit: int = 5
+
     parameters: Parameters = Parameters()
 
     def run(self, dataset: xr.Dataset, variable_name: str, failures: NDArray[np.bool8]):
-        if self._exceeds_tolerance(failures):
-            raise DataQualityError(
-                f"Quality results for variable {variable_name} indicate a fatal error"
-                " has occurred and the pipeline should exit. Manual review of the data"
-                " is recommended.\n"
-                f"Extra context: '{self.parameters.context}'\n"
-                f"Quality results array: {failures}"
+        if self._exceeds_tolerance(failures):  # default failure tolerance is 0%
+            msg = (
+                f"Quality results for variable '{variable_name}' indicate a fatal error"
+                " has occurred. Manual review of the data is recommended.\n"
             )
 
+            # Show % failed if tolerance is set
+            fail_rate: float = np.average(failures)  # type: ignore
+            msg += (
+                f" {np.count_nonzero(failures)} / {failures.size} values failed"  # type: ignore
+                f" ({100*fail_rate:.2f}%), exceeding the allowable threshold of"
+                f" {100*self.parameters.tolerance}%.\n"
+            )
+
+            # Want to show the first few indexes where the test failed and also the
+            # corresponding data values. Careful to not show too many, otherwise the
+            # message will be bloated and hard to read. Note that np.nonzero(failures)
+            # returns a hard-to-read tuple of indexes, so we modify that to be easier to
+            # read and show the first self.parameters.display_limit # of errors.
+            failed_where = np.nonzero(failures)  # type: ignore
+            failed_values = list(dataset[variable_name].values[failed_where][: self.parameters.display_limit])  # type: ignore
+            failed_indexes: Union[List[int], List[List[int]]]
+            if len(failed_where) == 1:  # 1D
+                failed_indexes = list(failed_where[0][: self.parameters.display_limit])
+            else:
+                failed_indexes = [
+                    [dim_idxs[i] for dim_idxs in failed_where]
+                    for i in range(
+                        min(self.parameters.display_limit, len(failed_where[0]))
+                    )
+                ]
+            msg += (
+                f"The first failures occur at indexes: {failed_indexes}. The"
+                f" corresponding values are: {failed_values}.\n"
+            )
+
+            raise DataQualityError(msg)
         return dataset
 
     def _exceeds_tolerance(self, failures: NDArray[np.bool8]) -> bool:
+        if self.parameters.tolerance == 0:
+            return bool(failures.any())
         failure_ratio: float = np.average(failures)  # type: ignore
         return failure_ratio > self.parameters.tolerance
 
@@ -94,6 +114,14 @@ class RecordQualityResults(QualityHandler):
         meaning: str
         """A string that describes the test applied."""
 
+        @validator("assessment", pre=True)
+        def to_lower(cls, assessment: Any) -> str:
+            if isinstance(assessment, str):
+                return assessment.lower()
+            raise ValueError(
+                f"assessment must be 'bad' or 'indeterminate', not {assessment}"
+            )
+
     parameters: Parameters
 
     def run(
@@ -135,6 +163,8 @@ class SortDatasetByCoordinate(QualityHandler):
         ascending: bool = True
         """Whether to sort the dataset in ascending order. Defaults to True."""
 
+        correction: str = "Coordinate data was sorted in order to ensure monotonicity."
+
     parameters: Parameters = Parameters()
 
     def run(
@@ -142,4 +172,7 @@ class SortDatasetByCoordinate(QualityHandler):
     ) -> xr.Dataset:
         if failures.any():
             dataset = dataset.sortby(variable_name, ascending=self.parameters.ascending)  # type: ignore
+            record_corrections_applied(
+                dataset, variable_name, self.parameters.correction
+            )
         return dataset
